﻿#ifndef VIDEONETCLIENTPACKAGE_H
#define VIDEONETCLIENTPACKAGE_H

#include "src/inc/aifacepch.h"
#include "src/inc/aifaceobject.h"
#include "src/inc/package/videonetcallback.h"
#include "src/inc/devicemanager/deviceanalysiser.h"
#include "src/inc/devicemanager/devicemanager.h"
#include "src/inc/usermanager/userinfo.h"

#include <atomic>
#include <thread>

namespace vncp
{

class VideoPackage : public QObject
{
	Q_OBJECT
public:
	/**
	 * @brief GetInstance		单例获取接口
	 * @return
	 */
	static VideoPackage *GetInstance()
	{
		static VideoPackage videoNetClientPackage;
		return &videoNetClientPackage;
	}

protected:
	VideoPackage();
	~VideoPackage();

public:
	/**
	 * @brief VideoNetClientPackage		禁用的拷贝构造函数
	 */
	VideoPackage(const VideoPackage &)=delete;
	/**
	 * @brief operator =		禁用的赋值运算符
	 * @return
	 */
	VideoPackage operator=(const VideoPackage &)=delete;
	/**
	 * @brief VideoNetClientPackage		禁用的移动构造函数
	 */
	VideoPackage(VideoPackage &&)=delete;

public:
	/**
	 * @brief setCallback		设置回调对象指针
	 * @param callback
	 */
	void setCallback(VideoNetClientCallback *callback);

	/**
	 * @brief startDeviceDetectService		启动设备探测服务
	 * @param beginPort							[in]起始端口
	 * @param endPort								[in]结束端口
	 * @param updateTime							[in]设备回调间隔时间(ms)
	 * @return
	 */
	int startDeviceDetectService(const int &beginPort=5555,const int &endPort=5555,const int &updateTime=1000);

	/**
	 * @brief stopDeviceDetectService		停止设备探测服务
	 * @return
	 */
	int stopDeviceDetectService();

	/**
	 * @brief loginDevice			登录设备
	 * @param mac					[in]mac地址
	 * @param ip						[in]ip地址
	 * @param port					[in]信令端口号
	 * @param username			[in]用户名
	 * @param password			[in]密码
	 * @param userID					[out]登录ID，以后用该ID操作设备
	 */
	int loginDevice(const QString &mac,const QString &ip,const int &port,const QString &username,const QString &password,int &userID);

	/**
	 * @brief logoutDevice			登出设备
	 * @param userID					[in]用户ID
	 */
	int logoutDevice(const int &userID);

	/**
	 * @brief subscribeDeviceEvent	订阅设备事件
	 * @param userID							[in]用户ID
	 * @param deviceEventType			[in]事件类型
	 * @return
	 */
	[[deprecated]]
	int subscribeDeviceEvent(const int &userID,const DeviceEventType &deviceEventType);

	/**
	 * @brief unSubscribeDeviceEvent		取消订阅设备事件
	 * @param userID									[in]用户ID
	 * @param deviceEventType					[in]事件类型
	 * @return
	 */
	[[deprecated]]
	int unSubscribeDeviceEvent(const int &userID,const DeviceEventType &deviceEventType);

	/**
	 * @brief subSubscribeFaceEvent		订阅人脸有关事件
	 * @param userID								[in]用户ID
	 * @return
	 */
	int subscribeFaceEvent(const int &userID);

	/**
	 * @brief unSubscribeFaceEvent		取消订阅人脸有关事件
	 * @param userID								[in]用户ID
	 * @return
	 */
	int unsubscribeFaceEvent(const int &userID);

	/**
	 * @brief subscribeBasicPeopleDetectEvent		订阅基础人形事件
	 * @param userID												[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int subscribeBasicPeopleDetectEvent(const int &userID);

	/**
	 * @brief unSubscribeBasePeopleDetectEvent		取消订阅基础人形事件
	 * @param userID													[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int unSubscribeBasePeopleDetectEvent(const int &userID);

	/**
	 * @brief subscribeBasicFaceDetectEvent			订阅基础人脸事件
	 * @param userID												[in]订阅基础人脸事件
	 * @return
	 */
	[[deprecated]]
	int subscribeBasicFaceDetectEvent(const int &userID);

	/**
	 * @brief unSubscribeBasicFaceDetectEvent			取消订阅基础人脸事件
	 * @param userID												[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int unSubscribeBasicFaceDetectEvent(const int &userID);

	/**
	 * @brief subscribeAdvancedFaceDetectEvent			订阅高级人脸检测事件
	 * @param userID														[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int subscribeAdvancedFaceDetectEvent(const int &userID);

	/**
	 * @brief unSubscribeAdvancedFaceDetectEvent			取消订阅高级人脸检测事件
	 * @param userID														[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int unSubscribeAdvancedFaceDetectEvent(const int &userID);

	/**
	 * @brief subscribeAdvancedFaceRecognizeEvent		订阅高级人脸识别事件
	 * @param userID														[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int subscribeAdvancedFaceRecognizeEvent(const int userID);

	/**
	 * @brief unSubscribeAdvancedFaceRecognizeEvent		取消订阅高级人脸识别事件
	 * @param userID														[in]用户ID
	 * @return
	 */
	[[deprecated]]
	int unSubscribeAdvancedFaceRecognizeEvent(const int userID);

	/**
	 * @brief startRealStream			开始实时流
	 * @param userID						[in]用户ID
	 * @param channelID				[in]通道ID
	 * @param widget						[in]窗体
	 * @return
	 */
	int startRealStream(const int &userID,const int &channelID,QWidget *widget);

	/**
	 * @brief stopRealStream			停止实时流
	 * @param userID						[in]用户ID
	 * @param channelID				[in]通道ID
	 * @return
	 */
	int stopRealStream(const int &userID,const int &channelID);

	/**
	 * @brief drawRectOnStream		在视频流上绘制矩形
	 * @param userID							[in]用户ID
	 * @param channelID					[in]通道ID
	 * @param rects							[in]矩形
	 * @return
	 */
	int addRectOnStream(const int &userID,const int &channelID,const QVector<QRect> &rects);

private:
	VideoNetClientCallback *m_callback{nullptr};

	std::atomic_bool m_deviceUpdateFlag{false};
	std::thread *m_deviceUpdater{nullptr};

private:
	template<typename callable, class... arguments>
	inline void asyncWait(long after, callable&& f, arguments&&... args)
	{
		std::function<typename std::result_of<callable(arguments...)>::type()> task
				(std::bind(std::forward<callable>(f), std::forward<arguments>(args)...));

		std::thread([after, task]() {
			std::this_thread::sleep_for(std::chrono::milliseconds(after));
			task();
		}).detach();
	}

signals:
	void signal_deviceUpdate(QVector<DeviceInfo>);
};

}

#endif // VIDEONETCLIENTPACKAGE_H
